/*
 *
 * NAME = EPSJLink
 *
 * DESCRIPTIVE NAME = CICS (Samples) Example Application -
 *                          Invoke CICS Events for WBE SupportPac
 *
 *  Licensed Materials - Property of IBM
 *
 *  "Restricted Materials of IBM"
 *
 *  CICS SupportPac CB11
 *
 *  (C) Copyright IBM Corporation 2008 All Rights Reserved
 *  US Government Users Restricted Rights - Use, duplication
 *  or disclosure restricted by GSA ADP Schedule Contract
 *  with IBM Corporation
 *
 * STATUS = CB11
 *
 * TRANSACTION NAME = as defined
 *
 * FUNCTION =
 *      This program provides an example of invoking the CICS
 *      Events for WBE SupportPac, providing event details to be
 *      formatted and emitted by the SupportPac program.
 *
 * CHANGE ACTIVITY :
 *
 *   PN= REASON REL YYMMDD HDXXIII : REMARKS
 *  $D0= I08012 310 081121 HDIKTG  : Create sample
 *  $D1= F25583 EUR 090325 HDIKTG  : Add upwards compatibility
 *  $P1= D23176 310 081203 HDAVCMM : Change queue name
 *
 */
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.ibm.cics.server.*;

import cicseps.*;
 
public class EPSJLink
{
    // @start_class_copyright@
    private static final String COPYRIGHT =
    "Licensed Materials - Property of IBM " +

    "5697-E93 " +

    "(c) Copyright IBM Corp. 2008 All Rights Reserved. " +

    "US Government Users Restricted Rights - Use, duplication or " +
    "disclosure restricted by GSA ADP Schedule Contract with " +
    "IBM Corp.";
    // @end_class_copyright@

    public static void main(java.lang.String[] args)
                       throws CicsConditionException
    {
        /*
         * This example program just sets up the event with some data
         * and emits it.
         * In a real-life example, the event data would be created as a
         * result of the business processing carried out by the program,
         * and quite possibly, more than one event would be emitted.
         */
        Task t = Task.getTask();           // Get the CICS Task object
        if (t == null) {
            writeErrorMessage("getTask() failed");
            return;
        }
        writeInfoMessage("Starting EPSJLink");

        // Create the EPS Event
        boolean compatible = true;                                             /* @D1A */
        EPSEvent ev = new EPSEvent("Order_Event", compatible);                 /* @D1C */

        // Set the Configuration data (WMQ queue name)
        ev.setConfig("EP.TEST.EMITQ");
        
        // Set the first Event data item - text by default
        ev.setEventData("OrderItem", "12345");

        // Set the second Event data item - numeric value
        ev.setEventData("OrderValue", 10);

        // Set the third Event data item - text by default
        ev.setEventData("Customer", "Someone Important");

        // Emit the Event
        try {
            ev.emit();
        } catch (EPSEventException eve) {
            writeErrorMessage("EPS Exception was thrown");
            eve.printStackTrace();
        }
        writeInfoMessage("Completing EPSJLink");
    }

    /**
     * Write an information message
     * @param insert the string to be inserted into the standard 
     *        message format "date time EPSJLink insert"
     */
    static void writeInfoMessage(String insert)
    {
        System.out.println(getDateTime() + " EPSJLink " + insert);
    }

    /**
     * Write an error message
     * @param insert the string to be inserted into the standard 
     *        message format "date time EPSJLink insert"
     */
    static void writeErrorMessage(String insert)
    {
        System.err.println(getDateTime() + " EPSJLink " + insert);
    }

    /**
     * Get the current data and time
     * @return the current date and time as a String in the format
     *         "MMddyyyy  HHmmss"
     */
    static String getDateTime()
    {
        DateFormat df = new SimpleDateFormat("MMddyyyy HHmmss");
        Date today = new Date();
        return df.format(today);
    }
}
