       CBL CICS('COBOL3') APOST
      ******************************************************************
      *                                                                *
      * MODULE NAME = EPS0LINK                                         *
      *                                                                *
      * DESCRIPTIVE NAME = CICS (Samples) Example Application -        *
      *                          Invoke CICS Events for WBE SupportPac *
      *                                                                *
      *  Licensed Materials - Property of IBM                          *
      *                                                                *
      *  "Restricted Materials of IBM"                                 *
      *                                                                *
      *  CICS SupportPac CB11                                          *
      *                                                                *
      *  (C) Copyright IBM Corporation 2008, 2009 All Rights Reserved  *
      *  US Government Users Restricted Rights - Use, duplication      *
      *  or disclosure restricted by GSA ADP Schedule Contract         *
      *  with IBM Corporation                                          *
      *                                                                *
      *                                                                *
      * STATUS = CB11                                                  *
      *                                                                *
      * TRANSACTION NAME = as defined                                  *
      *                                                                *
      * FUNCTION =                                                     *
      *      This program provides an example of invoking the CICS     *
      *      Events for WBE SupportPac, providing event details to be  *
      *      formatted and emitted by the SupportPac program.          *
      *                                                                *
      *----------------------------------------------------------------*
      *                                                                *
      * ENTRY POINT = EPS0LINK                                         *
      *                                                                *
      *----------------------------------------------------------------*
      *                                                                *
      * CHANGE ACTIVITY :                                              *
      *                                                                *
      *      MOD(EPS0LINK),COMP(SAMPLES),PROD(CB11):                  *
      *                                                                *
      *   PN= REASON REL YYMMDD HDXXIII : REMARKS                      *
      *  D0= I08012 310 080917 HDAVCMM : Create sample                *
      *  P1= D22980 310 081123 HDIKTG  : Fix COPY EPS0CONF error      *
      *  P2= D23176 310 081203 HDAVCMM : Change queue name in config  *
      *  P3= D23231 310 081205 HDAVCMM : Change some field sizes      *
      *  P4= D23245 310 081208 HDAVCMM : Amend error checking on LINK *
      *  P5= D24725 410 090223 HDAVCMM : Compatibility and migration  *
      *  P6= D26071 410 090423 HDAVCMM : Copyright years              *
      *                                                                *
      ******************************************************************
       IDENTIFICATION DIVISION.
       PROGRAM-ID. EPS0LINK.
       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       DATA DIVISION.
       WORKING-STORAGE SECTION.
      *----------------------------------------------------------------*
      * Common definitions                                             *
      *----------------------------------------------------------------*
 
      *================================================================*
      * This section of the sample program shows how you might create  *
      * definitions for the various container names and structures     *
      * used by the SupportPac.  Of course, you have complete freedom  *
      * over the variable names that you use.                          *
      *================================================================*
 
      *----------------------------------------------------------------*
      * CICS Events for WBE SupportPac program name
      *----------------------------------------------------------------*
        01  EPS-PROGRAM-NAME           PIC X(8)
                                        VALUE 'EPSEMIT '.
 
      *----------------------------------------------------------------*
      * CICS Events SupportPac standard channel and containers
      * This creates definitions for the standard names expected by
      * the SupportPac for the channel name and for the containers
      * within that channel.
      *
      * In this example, three pieces of event data are passed, each
      * in a pair of containers giving the data item's name and value.
      * To illustrate specification of data that WBE is to regard as
      * numeric, for one item of data a type is also passed.
      *----------------------------------------------------------------*
        01  EPS-CHANNEL                PIC X(16)
                                        VALUE 'EPS.EVENTCHANNEL'.
        01  EVENT-NAME-CONTAINER       PIC X(16)
                                        VALUE 'EPS.EVENTNAME   '.
        01  EVENT-CONFIG-CONTAINER      PIC X(16)
                                        VALUE 'EPS.CONFIG      '.
        01  EVENT-ERROR-CONTAINER      PIC X(16)
                                        VALUE 'EPS.ERROR       '.
        01  EVENT-DATA-NAME-1           PIC X(16)
                                        VALUE 'EPS.NAME.01     '.
        01  EVENT-DATA-VALUE-1         PIC X(16)
                                        VALUE 'EPS.DATA.01     '.
        01  EVENT-DATA-NAME-2           PIC X(16)
                                        VALUE 'EPS.NAME.02     '.
        01  EVENT-DATA-VALUE-2         PIC X(16)
                                        VALUE 'EPS.DATA.02     '.
        01  EVENT-DATA-NAME-3           PIC X(16)
                                        VALUE 'EPS.NAME.03     '.
        01  EVENT-DATA-VALUE-3         PIC X(16)
                                        VALUE 'EPS.DATA.03     '.
 
      * Variables for time/date processing (included in message)
       01  ABS-TIME                    PIC S9(8) COMP VALUE +0.
       01  TIME1                       PIC X(8)  VALUE SPACES.
       01  DATE1                       PIC X(10) VALUE SPACES.
 
      * Progress message issued by this program
       01  PROGRESS-MSG.
           03 EP-DATE                  PIC X(8)  VALUE SPACES.
           03 FILLER                   PIC X     VALUE SPACES.
           03 EP-TIME                  PIC X(6)  VALUE SPACES.
           03 FILLER                   PIC X(9)  VALUE ' EPS0LINK'.
           03 EP-DETAIL                PIC X(50) VALUE SPACES.
 
      * Error message issued by this program
       01  ERROR-MSG.
           03 EM-DATE                  PIC X(8)  VALUE SPACES.
           03 FILLER                   PIC X     VALUE SPACES.
           03 EM-TIME                  PIC X(6)  VALUE SPACES.
           03 FILLER                   PIC X(9)  VALUE ' EPS0LINK'.
           03 EM-DETAIL                PIC X(40) VALUE SPACES.
           03 EM-ERRINTRO              PIC X(11) VALUE SPACES.
           03 EM-ERRCODE               PIC X(8)  VALUE SPACES.
 
      *================================================================*
      * This section of the sample program creates definitions for the *
      * configuration information expected by the SupportPac program   *
      * in the EPS.CONFIG container, and the error information         *
      * returned by the SupportPac if an error occurs in the EPS.ERROR *
      * container.                                                     *
      *================================================================*
 
      *----------------------------------------------------------------*
      * Configuration container structure
      *----------------------------------------------------------------*
       COPY EPS0CONF.
 
      *----------------------------------------------------------------*
      * Definition describing the error structure returned by the
      * CICS Events SupportPac program.
      *----------------------------------------------------------------*
       COPY EPS0ERR.
 
      * Working Variables
       01 WORKING-VARIABLES.
           03 EVENT-NAME               PIC X(32) VALUE SPACES.
           03 EVENT-DATA-NAME          PIC X(16) VALUE SPACES.
           03 EVENT-DATA-VALUE         PIC X(20) VALUE SPACES.
           03 RESP                     PIC S9(8) COMP.
           03 RESP2                    PIC S9(8) COMP.
 
      *----------------------------------------------------------------*
 
      ******************************************************************
      *    L I N K A G E   S E C T I O N
      ******************************************************************
       LINKAGE SECTION.
      * No commarea used by this example.
      * Other definition structures might be included.
 
      ******************************************************************
      *    P R O C E D U R E S
      ******************************************************************
       PROCEDURE DIVISION.
 
      *----------------------------------------------------------------*
       MAINLINE SECTION.
 
      *================================================================*
      * This example program just sets up the containers with some     *
      * event data and links to the SupportPac program to emit an      *
      * an event.                                                      *
      * In a real-life example, the event data would be created as a   *
      * result of the business processing carried out by the program,  *
      * and quite possibly, more than one event would be emitted.      *
      *================================================================*
 
      *----------------------------------------------------------------*
      * Common code                                                    *
      *----------------------------------------------------------------*
      * initialize working storage variables
           INITIALIZE PROGRESS-MSG
           INITIALIZE ERROR-MSG
           INITIALIZE EPS-CONFIG
           INITIALIZE WORKING-VARIABLES
 
           MOVE ' Starting EPS0LINK' TO EP-DETAIL
           PERFORM WRITE-MESSAGE
 
      *---------------------------------------------------------------*
      * Set up the data for the Event
      *---------------------------------------------------------------*
 
      *---------------------------------------------------------------*
      * Place the event name into the appropriate container in the
      * channel via the event name variable
      *---------------------------------------------------------------*
           MOVE 'Order_Event' TO EVENT-NAME
           EXEC CICS PUT CONTAINER(EVENT-NAME-CONTAINER)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EVENT-NAME)
           END-EXEC
 
      *---------------------------------------------------------------*
      * Place the queue name into the appropriate container in the
      * channel via the queue name variable
      * Queue names should be padded to the right to 48 chars
      * which should be the case here as it was init with spaces.
      * Use the compatibility indicator to request upwards
      * compatibility checking to ease migration to new event support
      *---------------------------------------------------------------*
           MOVE 'EP.TEST.EMITQ' TO CON-QUEUE-NAME
           MOVE 'ON' TO CON-COMPAT
           EXEC CICS PUT CONTAINER(EVENT-CONFIG-CONTAINER)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EPS-CONFIG)
           END-EXEC
 
      *---------------------------------------------------------------*
      * Place the first item of data and its name into the appropriate
      * pair of containers in the channel
      * Clear the working variables, which include the data name and
      * value, ready for the next pair
      *---------------------------------------------------------------*
           MOVE 'OrderItem' TO EVENT-DATA-NAME
           EXEC CICS PUT CONTAINER(EVENT-DATA-NAME-1)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EVENT-DATA-NAME)
           END-EXEC
           MOVE '12345     ' TO EVENT-DATA-VALUE
           EXEC CICS PUT CONTAINER(EVENT-DATA-VALUE-1)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EVENT-DATA-VALUE)
           END-EXEC
           INITIALIZE WORKING-VARIABLES
 
      *---------------------------------------------------------------*
      * Place the second item of data and its name into the appropriate
      * pair of containers in the channel
      * Also place an indication that this data item is to be treated
      * as numeric by WBE, by placing a type of 'numeric' in the TYPE
      * container for this item of data
      * Clear the working variables, which include the data name and
      * value, ready for the next pair
      *---------------------------------------------------------------*
      * This data item is to be used as numeric, but that can be
      * specified in the WBE tooling, so we do not pass any data type
      *---------------------------------------------------------------*
           MOVE 'OrderValue' TO EVENT-DATA-NAME
           EXEC CICS PUT CONTAINER(EVENT-DATA-NAME-2)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EVENT-DATA-NAME)
           END-EXEC
           MOVE '10' TO EVENT-DATA-VALUE
           EXEC CICS PUT CONTAINER(EVENT-DATA-VALUE-2)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EVENT-DATA-VALUE)
           END-EXEC
           INITIALIZE WORKING-VARIABLES
 
      *---------------------------------------------------------------*
      * Place the third item of data and its name into the appropriate
      * pair of containers in the channel
      *---------------------------------------------------------------*
           MOVE 'Customer' TO EVENT-DATA-NAME
           EXEC CICS PUT CONTAINER(EVENT-DATA-NAME-3)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EVENT-DATA-NAME)
           END-EXEC
           MOVE 'Someone Important' TO EVENT-DATA-VALUE
           EXEC CICS PUT CONTAINER(EVENT-DATA-VALUE-3)
                         CHANNEL(EPS-CHANNEL)
                         FROM(EVENT-DATA-VALUE)
           END-EXEC
 
      *---------------------------------------------------------------*
      * Link to the SupportPac program to emit the event
      *---------------------------------------------------------------*
 
           EXEC CICS LINK PROGRAM(EPS-PROGRAM-NAME)
                     CHANNEL(EPS-CHANNEL)
                     RESP(RESP) RESP2(RESP2)
           END-EXEC.
 
      *===============================================================*
      * Error checking                                                *
      *===============================================================*
 
      * Check the return code was normal
 
           EVALUATE RESP
               WHEN DFHRESP(NORMAL)
                   CONTINUE
 
               WHEN DFHRESP(INVREQ)
                   MOVE ' INVREQ on LINK to EPSEMIT' TO EM-DETAIL
                   PERFORM WRITE-ERROR-MESSAGE
 
               WHEN DFHRESP(PGMIDERR)
                   MOVE ' PGMIDERR attempting to link to EPSEMIT'
                       TO EM-DETAIL
                   PERFORM WRITE-ERROR-MESSAGE
 
               WHEN OTHER
                   MOVE ' Error linking to EPSEMIT' to EM-DETAIL
                   PERFORM WRITE-ERROR-MESSAGE
           END-EVALUATE.
 
      *================================================================*
      * Check whether an ERROR container has been returned indicating  *
      * that an error occurred within the SupportPac program.          *
      * Check the ERROR container for problems occurring when the      *
      * CICS Events SupportPac attempted to emit the event.            *
      *================================================================*
 
      * Do we have an error container?
           EXEC CICS GET CONTAINER(EVENT-ERROR-CONTAINER)
                     INTO(EVENT-ERROR-STRUCTURE)
                     CHANNEL(EPS-CHANNEL)
                     RESP(RESP) RESP2(RESP2)
           END-EXEC
 
      *----------------------------------------------------------------*
      * Check response from GET CONTAINER.  NORMAL response means
      * we have an ERROR container and can process the error info.
      * CONTAINERERR means no ERROR container, so no errors from
      * SupportPac program.  Other errors mean we could not check
      * for the error container for some reason.
      *----------------------------------------------------------------*
           IF RESP EQUAL DFHRESP(NORMAL) THEN
             EVALUATE TRUE
                 WHEN ERR-ERROR-CODE IS EQUAL TO 0
                   CONTINUE
                 WHEN ERR-ERROR-CODE IS GREATER THAN 90
                   MOVE ' Warning issued by EPSEMIT' TO EM-DETAIL
                   MOVE ' Warning = ' TO EM-ERRINTRO
                   MOVE ERR-ERROR-CODE TO EM-ERRCODE
                   PERFORM WRITE-ERROR-MESSAGE
                 WHEN OTHER
                   MOVE ' Error occurred in EPSEMIT' TO EM-DETAIL
                   MOVE ' ErrCode = ' TO EM-ERRINTRO
                   MOVE ERR-ERROR-CODE TO EM-ERRCODE
                   PERFORM WRITE-ERROR-MESSAGE
             END-EVALUATE
           ELSE
             IF RESP NOT EQUAL DFHRESP(CONTAINERERR) THEN
               MOVE ' Unable to check for errors from EPSEMIT'
                    TO EM-DETAIL
               PERFORM WRITE-ERROR-MESSAGE
             END-IF
           END-IF.
 
      * Send a message to say we are done
           INITIALIZE PROGRESS-MSG
           MOVE ' Completing EPS0LINK' TO EP-DETAIL
           PERFORM WRITE-MESSAGE
 
      * Return to caller
           EXEC CICS RETURN END-EXEC.
 
       MAINLINE-EXIT.
           EXIT.
      *----------------------------------------------------------------*
 
      *================================================================*
      * Procedure to write message on progress of this program to CSMT *
      *   Message will include Date, Time, Program Name,               *
      *   and information about current progress.                      *
      *================================================================*
       WRITE-MESSAGE.
      * Obtain and format current time and date
           EXEC CICS ASKTIME ABSTIME(ABS-TIME)
           END-EXEC
           EXEC CICS FORMATTIME ABSTIME(ABS-TIME)
                     MMDDYYYY(DATE1)
                     TIME(TIME1)
           END-EXEC
           MOVE DATE1 TO EP-DATE
           MOVE TIME1 TO EP-TIME
      * Write output message
           EXEC CICS WRITEQ TD QUEUE('CSMT')
                     FROM(PROGRESS-MSG)
                     LENGTH(LENGTH OF PROGRESS-MSG)
           END-EXEC.
           EXIT.
 
      *================================================================*
      * Procedure to write error message to TD queue CSMT.             *
      *   Message will include Date, Time, Program Name,               *
      *   and error details.                                           *
      *================================================================*
       WRITE-ERROR-MESSAGE.
      * Obtain and format current time and date
           EXEC CICS ASKTIME ABSTIME(ABS-TIME)
           END-EXEC
           EXEC CICS FORMATTIME ABSTIME(ABS-TIME)
                     MMDDYYYY(DATE1)
                     TIME(TIME1)
           END-EXEC
           MOVE DATE1 TO EM-DATE
           MOVE TIME1 TO EM-TIME
      * Write output message
           EXEC CICS WRITEQ TD QUEUE('CSMT')
                     FROM(ERROR-MSG)
                     LENGTH(LENGTH OF ERROR-MSG)
           END-EXEC.
           EXIT.
